<?php
/**
 * URL manipulation functions
 *
 * @uses idna_convert
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Check the validity of a given URL
 *
 * Checks allowed URI schemes and URL ban list. URL can be empty. Note: We have a
 * problem when trying to "antispam" a keyword which is already blacklisted. If
 * that keyword appears in the URL then the next page has a bad referer!
 *
 * @uses antispam_check()
 * @param string URL to validate
 * @param string Context ("posting", "commenting")
 * @param boolean also do an antispam check on the url
 * @param string String for counter to increment (comments, referers, messages),
 * or false if we do not want/need increment any counter
 * @return mixed false (which means OK) or error message
 */
function validate_url( $url, $context = 'posting', $antispam_check = true, $store_as = false )
{
	global $debug;

	if( empty( $url ) )
	{
		// Empty URL, no problem
		return false;
	}

	$verbose = $debug || $context != 'commenting';

	$allowed_uri_schemes = get_allowed_uri_schemes( $context );

	// Validate URL structure
	if( $url[0] == '$' )
	{
		// This is a 'special replace code' URL (used in footers)
		if( ! preg_match( '\$([a-z_]+)\$', $url ) )
		{
			return T_('Invalid URL $code$ format');
		}
	}
	elseif( preg_match( '~^\w+:~', $url ) )
	{
		// there's a scheme and therefore an absolute URL:
		if( substr( $url, 0, 7 ) == 'mailto:' )
		{
			// mailto:link
			if( ! in_array( 'mailto', $allowed_uri_schemes ) )
			{
				// Scheme not allowed
				$scheme = 'mailto:';
				return $verbose
					? sprintf( T_('URI scheme "%s" not allowed.'), htmlspecialchars( $scheme ) )
					: T_('URI scheme not allowed.');
			}

			preg_match( '~^(mailto):(.*?)(\?.*)?$~', $url, $match );
			if( ! $match )
			{
				return $verbose
					? sprintf( T_('Invalid email link: %s.'), htmlspecialchars( $url ) )
					: T_('Invalid email link.');
			}
			elseif( ! is_email( $match[2] ) )
			{
				return $verbose
					? sprintf( T_('Supplied email address (%s) is invalid.'), htmlspecialchars( $match[2] ) )
					: T_('Invalid email address.');
			}
		}
		elseif( substr( $url, 0, 6 ) == 'clsid:' )
		{
			// clsid:link
			if( ! in_array( 'clsid', $allowed_uri_schemes ) )
			{
				// Scheme not allowed
				$scheme = 'clsid:';
				return $verbose
					? sprintf( T_('URI scheme "%s" not allowed.'), htmlspecialchars( $scheme ) )
					: T_('URI scheme not allowed.');
			}

			if( ! preg_match( '^(clsid):([a-fA-F0-9\-]+)$', $url, $match ) )
			{
				return T_('Invalid class ID format');
			}
		}
		elseif( substr( $url, 0, 11 ) == 'javascript:' )
		{
			// javascript ... basically there could be anything here
			if( ! in_array( 'javascript', $allowed_uri_schemes ) )
			{
				// Scheme not allowed
				$scheme = 'javascript:';
				return $verbose
					? sprintf( T_('URI scheme "%s" not allowed.'), htmlspecialchars( $scheme ) )
					: T_('URI scheme not allowed.');
			}

			preg_match( '^(javascript):', $url, $match );
		}
		else
		{
			// convert URL to IDN
			$url = idna_encode( $url );

			if( ! preg_match('~^               # start
				([a-z][a-z0-9+.\-]*)           # scheme
				://                            # authorize absolute URLs only ( // not present in clsid: -- problem? ; mailto: handled above)
				(\w+(:\w+)?@)?                 # username or username and password (optional)
				( localhost |
						[a-z0-9]([a-z0-9\-])*  # Don t allow anything too funky like entities
						\.                     # require at least 1 dot
						[a-z0-9]([a-z0-9.\-])+ # Don t allow anything too funky like entities
				)
				(:[0-9]+)?                     # optional port specification
				.*                             # allow anything in the path (including spaces - used in FileManager - but no newlines).
				$~ix', $url, $match ) )
			{
				// Cannot validate URL structure
				return $verbose
					? sprintf( T_('Invalid URL format (%s).'), htmlspecialchars( $url ) )
					: T_('Invalid URL format.');
			}

			$scheme = strtolower( $match[1] );
			if( ! in_array( $scheme, $allowed_uri_schemes ) )
			{
				// Scheme not allowed
				return $verbose
					? sprintf( T_('URI scheme "%s" not allowed.'), htmlspecialchars( $scheme ) )
					: T_('URI scheme not allowed.');
			}
		}
	}
	else
	{
		// URL is relative
		if( $context == 'commenting' )
		{
			// We do not allow relative URLs in comments
			return $verbose
				? sprintf( T_('URL "%s" must be absolute.'), htmlspecialchars( $url ) )
				: T_('URL must be absolute.');
		}

		$char = substr( $url, 0, 1 );
		if( $char != '/' && $char != '#' )
		{
			// must start with a slash or hash (for HTML anchors to the same page)
			return $verbose
				? sprintf( T_('URL "%s" must be a full path starting with "/" or an anchor starting with "#".'), htmlspecialchars( $url ) )
				: T_('URL must be a full path starting with "/" or an anchor starting with "#".');
		}
	}

	if( $antispam_check )
	{
		// Search for blocked keywords
		if( $block = antispam_check( $url, $store_as ) )
		{
			return $verbose
				? sprintf( T_('URL "%s" not allowed: blacklisted word "%s".'), htmlspecialchars( $url ), $block )
				: T_('URL not allowed');
		}
	}

	return false; // OK
}


/**
 * Get allowed URI schemes for a given context
 *
 * @param string Context ("posting", "commenting")
 * @return array
 */
function get_allowed_uri_schemes( $context = 'posting' )
{
	global $current_User;

	$schemes = array(
		'http',
		'https',
		'ftp',
		'gopher',
		'nntp',
		'news',
		'mailto',
		'irc',
		'aim',
		'icq'
	);

	if( $context == 'commenting' )
	{
		return $schemes;
	}

	if( ! empty( $current_User ) )
	{
		// Add additional permissions the current User may have
		$Group = & $current_User->get_Group();

		if( $Group->perm_xhtml_javascript )
		{
			$schemes[] = 'javascript';
		}

		if( $Group->perm_xhtml_objects )
		{
			$schemes[] = 'clsid';
		}
	}
	return $schemes;
}


/**
 * Get the last HTTP status code received by the HTTP/HTTPS wrapper of PHP
 *
 * @param array The $http_response_header array (by reference).
 * @return integer|boolean False if no HTTP status header could be found, the
 * HTTP status code otherwise.
 */
function _http_wrapper_last_status( & $headers )
{
	for( $i = count( $headers ) - 1; $i >= 0; --$i )
	{
		if( preg_match( '|^HTTP/\d+\.\d+ (\d+)|', $headers[$i], $matches ) )
		{
			return $matches[1];
		}
	}
	return false;
}


/**
 * Fetch remote page
 *
 * Attempt to retrieve a remote page using a HTTP GET request, first with cURL,
 * then fsockopen, then fopen.
 *
 * @param string URL
 * @param array Info (by reference)
 * - 'error': holds error message, if any
 * - 'status': HTTP status (e.g. 200 or 404)
 * - 'used_method': Used method ("curl", "fopen", "fsockopen" or null if no methodis available)
 * @todo (0000) Should we try remaining methods, if the previous one(s) failed?
 * @todo (0000) Also allow HTTP POST.
 * @param integer Timeout
 * @return string|false The remote page as a string; false in case of error
 */
function fetch_remote_page( $url, & $info, $timeout = 15 )
{
	$info = array(
		'error' => '',
		'status' => NULL,
		'used_method' => NULL,
	);

	if( extension_loaded( 'curl' ) )
	{
		// CURL
		$info['used_method'] = 'curl';

		$ch = curl_init();
		curl_setopt( $ch, CURLOPT_URL, $url );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, $timeout );
		curl_setopt( $ch, CURLOPT_TIMEOUT, $timeout );
		$r = curl_exec( $ch );

		$info['status'] = curl_getinfo( $ch, CURLINFO_HTTP_CODE );
		$info['error'] = curl_error( $ch );
		if( ( $errno = curl_errno( $ch ) ) )
		{
			$info['error'] .= ' (#'.$errno.')';
		}
		curl_close( $ch );

		return $r;
	}
	elseif( function_exists( 'fsockopen' ) ) // may have been disabled
	{
		// FSOCKOPEN
		$info['used_method'] = 'fsockopen';

		if( ( $url_parsed = @parse_url( $url ) ) === false
			|| ! isset( $url_parsed['host'] ) )
		{
			$info['error'] = 'Could not parse URL';
			return false;
		}

		$host = $url_parsed['host'];
		$port = empty( $url_parsed['port'] ) ? 80 : $url_parsed['port'];
		$path = empty( $url_parsed['path'] ) ? '/' : $url_parsed['path'];
		if( ! empty( $url_parsed['query'] ) )
		{
			$path .= '?'.$url_parsed['query'];
		}

		$out = 'GET '.$path.' HTTP/1.1'."\r\n";
		$out .= 'Host: '.$host;
		if( ! empty( $url_parsed['port'] ) )
		{
			// we don't want to add :80 if not specified. remote end may not resolve it
			$out .= ':'.$port;
		}
		$out .= "\r\n".'Connection: Close'."\r\n\r\n";

		$fp = @fsockopen( $host, $port, $errno, $errstr, $timeout );
		if( ! $fp )
		{
			$info['error'] = $errstr.' (#'.$errno.')';
			return false;
		}

		// Send request
		fwrite( $fp, $out );

		// Set timeout for data
		if( function_exists( 'stream_set_timeout' ) )
		{
			stream_set_timeout( $fp, $timeout ); // PHP 4.3.0
		}
		else
		{
			socket_set_timeout( $fp, $timeout ); // PHP 4
		}

		// Read response
		$r = '';
		// First line
		$s = fgets( $fp );
		if( ! preg_match( '~^HTTP/\d+\.\d+ (\d+)~', $s, $match ) )
		{
			$info['error'] = 'Invalid response.';
			fclose( $fp );
			return false;
		}
		$info['status'] = $match[1];

		while( ! feof( $fp ) )
		{
			$r .= fgets( $fp );
		}
		fclose( $fp );

		if( ( $pos = strpos( $r, "\r\n\r\n" ) ) === false )
		{
			$info['error'] = 'Could not locate end of headers';
			return false;
		}

		return substr( $r, $pos + 4 );
	}
	elseif( ini_get( 'allow_url_fopen' ) )
	{
		// URL FOPEN
		$info['used_method'] = 'fopen';

		$fp = @fopen( $url, 'r' );
		if( ! $fp )
		{
			if( isset( $http_response_header )
				&& ( $code = _http_wrapper_last_status( $http_response_header ) ) !== false )
			{
				// fopen() returned false because it got a bad HTTP code
				$info['error'] = 'Invalid response';
				$info['status'] = $code;
				return '';
			}

			$info['error'] = 'fopen() failed';
			return false;
		}
		// Check just to be sure
		elseif( ! isset( $http_response_header )
			|| ( $code = _http_wrapper_last_status( $http_response_header ) ) === false )
		{
			$info['error'] = 'Invalid response';
			$r = false;
		}
		else
		{
			$info['status'] = $code;
			$r = '';
			while( ! feof( $fp ) )
			{
				$r .= fgets( $fp );
			}
		}

		fclose( $fp );
		return $r;
	}

	// All failed
	$info['error'] = 'No method available to access URL!';
	return false;
}


/**
 * Get $url with the same protocol (http/https) as $other_url
 *
 * @param string URL
 * @param string other URL (defaults to {@link $ReqHost})
 * @return string
 */
function url_same_protocol( $url, $other_url = NULL )
{
	global $ReqHost;

	if( is_null( $other_url ) )
	{
		$other_url = $ReqHost;
	}

	// change protocol of $url to same of admin ('https' <=> 'http')
	if( substr( $url, 0, 7 ) == 'http://' )
	{
		if( substr( $other_url, 0, 8 ) == 'https://' )
		{
			$url = 'https://'.substr( $url, 7 );
		}
	}
	elseif( substr( $url, 0, 8 ) == 'https://' )
	{
		if( substr( $other_url, 0, 7 ) == 'http://' )
		{
			$url = 'http://'.substr( $url, 8 );
		}
	}

	return $url;
}


/**
 * Add param(s) at the end of an URL
 *
 * Using either "?" or "&amp;" depending on existing url
 *
 * @param string existing url
 * @param string params to add
 * @param string delimiter to use for more params
 */
function url_add_param( $url, $param, $glue = '&amp;' )
{
	if( empty( $param ) )
	{
		return $url;
	}

	if( ( $anchor_pos = strpos( $url, '#' ) ) !== false )
	{
		// There's an "#anchor" in the URL
		$anchor = substr( $url, $anchor_pos );
		$url = substr( $url, 0, $anchor_pos );
	}
	else
	{
		// URL without "#anchor"
		$anchor = '';
	}

	if( strpos( $url, '?' ) !== false )
	{
		// There are already params in the URL
		$r = $url;
		if( substr( $url, -1 ) != '?' )
		{
			// the "?" is not the last char
			$r .= $glue;
		}
		return $r.$param.$anchor;
	}

	// These are the first params
	return $url.'?'.$param.$anchor;
}


/**
 * Add a tail (starting with "/") at the end of an URL before any params
 *
 * @param string existing url
 * @param string tail to add
 */
function url_add_tail( $url, $tail )
{
	$parts = explode( '?', $url );
	if( substr( $parts[0], -1 ) == '/' )
	{
		$parts[0] = substr( $parts[0], 0, -1 );
	}
	if( isset( $parts[1] ) )
	{
		return $parts[0].$tail.'?'.$parts[1];
	}

	return $parts[0].$tail;
}


/**
 * Try to make $url relative to $target_url, if scheme, host, user and pass matches
 *
 * This is useful for redirect_to params, to keep them short and avoid mod_security
 * rejecting the request as "Not Acceptable" (whole URL as param).
 *
 * NOTE: parse_url cannot handle URLs without scheme (e.g. "//example.com/foo").
 *
 * @param string URL to handle
 * @param string URL where we want to make $url relative to
 * @return string
 */
function url_rel_to_same_host( $url, $target_url )
{
	$parsed_url = @parse_url( $url );
	if( ! $parsed_url )
	{
		// invalid url
		return $url;
	}
	if( empty( $parsed_url['scheme'] ) || empty( $parsed_url['host'] ) )
	{
		// no protocol or host information
		return $url;
	}

	$target_url = @parse_url( $target_url );
	if( ! $target_url )
	{
		// invalid url
		return $url;
	}
	if( ! empty( $target_url['scheme'] ) && $target_url['scheme'] != $parsed_url['scheme'] )
	{
		// scheme/protocol is different
		return $url;
	}
	if( ! empty( $target_url['host']) )
	{
		if( empty( $target_url['scheme'] ) || $target_url['host'] != $parsed_url['host'] )
		{
			// target has no scheme (but a host) or hosts differ
			return $url;
		}

		if( @$target_url['port'] != @$parsed_url['port'] )
		{
			return $url;
		}
		if( @$target_url['user'] != @$parsed_url['user'] )
		{
			return $url;
		}
		if( @$target_url['pass'] != @$parsed_url['pass'] )
		{
			return $url;
		}
	}

	// We can make the URL relative
	$r = '';
	if( ! empty( $parsed_url['path'] ) )
	{
		$r .= $parsed_url['path'];
	}
	if( ! empty( $parsed_url['query'] ) )
	{
		$r .= '?'.$parsed_url['query'];
	}
	if( ! empty( $parsed_url['fragment'] ) )
	{
		$r .= '?'.$parsed_url['fragment'];
	}

	return $r;
}


/**
 * Make an $url absolute according to $host, if it is not absolute yet
 *
 * @param string URL
 * @param string Host (including protocol, e.g. 'http://example.com'); defaults
 * to {@link $ReqHost}
 * @return string
 */
function url_absolute( $url, $host = NULL )
{
	global $ReqHost;

	if( is_absolute_url( $url ) )
	{
		// URL is already absolute
		return $url;
	}

	if( empty( $host ) )
	{
		$host = $ReqHost;
	}
	return $host.$url;
}


/**
 * Make links in $s absolute
 *
 * It searches for "src" and "href" HTML tag attributes and makes the absolute.
 *
 * @uses url_absolute()
 * @param string content
 * @param string Hostname including scheme, e.g. http://example.com; defaults to $ReqHost
 * @return string
 */
function make_rel_links_abs( $s, $host = NULL )
{
	global $ReqHost;

	if( empty( $host ) )
	{
		$host = $ReqHost;
	}

	$s = preg_replace_callback( '~(<[^>]+?)\b((?:src|href)\s*=\s*)(["\'])?([^\\3]+?)(\\3)~i', create_function( '$m', '
		return $m[1].$m[2].$m[3].url_absolute( $m[4], "'.$host.'" ).$m[5];' ), $s );
	return $s;
}


/**
 * Display an URL, constrained to a max length
 *
 * @param string
 * @param integer
 */
function disp_url( $url, $max_length = NULL )
{
	if( ! empty( $max_length ) && strlen( $url ) > $max_length )
	{
		$disp_url = htmlspecialchars(substr( $url, 0, $max_length-1 )).'&hellip;';
	}
	else
	{
		$disp_url = htmlspecialchars( $url );
	}
	echo '<a href="'.$url.'">'.$disp_url.'</a>';
}


/**
 * Is a given URL absolute?
 *
 * Note: "//foo/bar" is absolute - leaving the protocol out.
 *
 * @param string URL
 * @return boolean
 */
function is_absolute_url( $url )
{
	return (bool)preg_match( '~^(?:\w+:)?//~', $url );
}


/**
 * Compare two given URLs, if they are the same
 *
 * This converts all urlencoded chars (e.g. "%AA") to lowercase.
 * It appears that some webservers use lowercase for the chars (Apache),
 * while others use uppercase (lighttpd).
 *
 * @return boolean
 */
function is_same_url( $a, $b )
{
	$a = preg_replace_callback( '~%[0-9A-F]{2}~', create_function( '$m', 'return strtolower( $m[0] );' ), $a );
	$b = preg_replace_callback( '~%[0-9A-F]{2}~', create_function( '$m', 'return strtolower( $m[0] );' ), $b );
	return $a == $b;
}


/**
 * IDNA-Encode URL to Punycode
 *
 * @param string URL
 * @return string Encoded URL (ASCII)
 */
function idna_encode( $url )
{
	global $app_charset;

	$url_utf8 = convert_charset( $url, 'utf-8', $app_charset );
	$IDNA = new idna_convert();
	$url = $IDNA->encode( $url_utf8 );
	/*
	if( $idna_error = $IDNA->get_last_error() )
	{
		echo $idna_error;
	}
	*/
	return $url;
}


/**
 * Decode IDNA puny-code ("xn--..") to UTF-8 name
 *
 * @param string
 * @return string The decoded puny-code ("xn--..") (UTF8!)
 */
function idna_decode( $url )
{
	$IDNA = new idna_convert();
	return $IDNA->decode( $url );
}

?>
