<?php
/**
 * This file implements the Timer base class
 *
 * This is a simple class to allow timing/profiling of code portions.
 *
 * @todo (1111) vars type 'foo' need useful/valid descriptions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package core
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Timer base class
 *
 * @package core
 */
class Timer
{
	/**
	 * Remember times
	 *
	 * We store for each category (primary key) the state, start/resume time and
	 * the total passed time.
	 *
	 * @var foo
	 */
	var $_times = array();

	/**
	 * Constructor
	 *
	 * @param string|NULL If a category is given the timer starts right away.
	 */
	function Timer( $category = NULL )
	{
		if( is_string( $category ) )
		{
			$this->start( $category );
		}
	}


	/**
	 * Reset a timer category
	 */
	function reset( $category )
	{
		$this->_times[$category] = array( 'total' => 0, 'count' => 0 );
	}


	/**
	 * Start a timer
	 */
	function start( $category, $log = true )
	{
		$this->reset( $category );
		$this->resume( $category );
	}


	/**
	 * Stops a timer category
	 *
	 * The timer may me resumed later on, see {@link resume()}. This is an alias
	 * for {@link pause()}.
	 *
	 * @return boolean false, if the timer had not been started.
	 */
	function stop( $category )
	{
		if( ! $this->pause( $category ) )
			return false;

		return true;
	}


	/**
	 * Pause a timer category
	 *
	 * It may me resumed later on, see {@link resume()}. NOTE: The timer needs to be
	 * started, either through the {@link Timer() Constructor} or the {@link start()}
	 * method.
	 *
	 * @return boolean false, if the timer had not been started.
	 */
	function pause( $category )
	{
		if( ! isset( $this->_times[$category]['resumed'] ) )
		{
			// Timer has not been started!
			return false;
		}
		$since_resume = $this->get_current_microtime() - $this->_times[$category]['resumed'];
		$this->_times[$category]['total'] += $since_resume;
		$this->_times[$category]['state'] = 'paused';

		return true;
	}


	/**
	 * Resume the timer on a category
	 */
	function resume( $category )
	{
		if( ! isset( $this->_times[$category]['total'] ) )
		{
			$this->start( $category );
		}
		else
		{
			$this->_times[$category]['resumed'] = $this->get_current_microtime();
			$this->_times[$category]['count']++;
			$this->_times[$category]['state'] = 'running';
		}
	}


	/**
	 * Get the duration for a given category
	 *
	 * @param string Category name
	 * @param integer Number of decimals after dot.
	 * @return string
	 */
	function get_duration( $category, $decimals = 3 )
	{
		// @todo (0000) decimals/seperator by locale!
		return number_format( $this->get_microtime( $category ), $decimals );
	}


	/**
	 * Get number of timer resumes (includes start)
	 *
	 * @return integer
	 */
	function get_count( $category )
	{
		if( isset( $this->_times[$category] ) )
		{
			return $this->_times[$category]['count'];
		}

		return false;
	}


	/**
	 * Get the time in microseconds that was spent in the given category
	 *
	 * @return float
	 */
	function get_microtime( $category )
	{
		switch( $this->get_state( $category ) )
		{
			case 'running':
			// The timer is running, we need to return the additional time since the last resume
			return $this->_times[$category]['total'] + $this->get_current_microtime() - $this->_times[$category]['resumed'];

			case 'paused':
			return $this->_times[$category]['total'];

			default:
			return (float)0;
		}
	}


	/**
	 * Get the state a category timer is in
	 *
	 * @return string 'unknown', 'not initialised', 'running', 'paused'
	 */
	function get_state( $category )
	{
		if( ! isset( $this->_times[$category] ) )
		{
			return 'unknown';
		}

		if( ! isset( $this->_times[$category]['state'] ) )
		{
			return 'not initialised';
		}

		return $this->_times[$category]['state'];
	}


	/**
	 * Get a list of used categories
	 *
	 * @return array
	 */
	function get_categories()
	{
		return array_keys( $this->_times );
	}


	/**
	 * Get the current time in microseconds.
	 *
	 * @return float
	 */
	function get_current_microtime()
	{
		list( $usec, $sec ) = explode( ' ', microtime() );
		return ( (float)$usec + (float)$sec );
	}

}

?>
