<?php
/**
 * This file implements Antispam handling functions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package antispam
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Insert a new abuse string into DB
 *
 * @uses antispam_check()
 * @param string "keyword" (domain) to be added to antispam table
 * @param string source of the keyword ('core','comment','entered','referer','shared')
 * @param boolean true (1) if the keyword has been shortened
 * @param integer number of comments removed by this keyword
 * @param integer number of referers removed by this keyword
 * @return string|false the pattern to match this domain in the blacklist; false if we
 * could not extract the base domain
 */
function antispam_create( $abuse_string, $source = 'core', $shortened = 0, $comments = 0, $referers = 0 )
{
	global $DB;
	global $localtimenow;

	// Get out if the string is empty
	$abuse_string = trim( $abuse_string );
	if( empty( $abuse_string ) )
	{
		return false;
	}

	// Get out if the string already is in the blacklist
	if( antispam_check( $abuse_string ) )
	{
		return false;
	}

	// Insert new string into DB
	$added_on = date( 'Y-m-d H:i:s', $localtimenow );
	$lastseen = date( 'Y-m-d H:i:s', $localtimenow );
	$sql = "INSERT INTO T_antispam(
		aspm_string, aspm_source, aspm_added_on, aspm_lastseen, aspm_shortened, aspm_comments, aspm_referers ) VALUES (
		'".$DB->escape( $abuse_string )."', '".$DB->escape( $source )."', '".$DB->escape( $added_on )."',
		'".$DB->escape( $lastseen )."', $shortened, $comments, $referers )";
	$DB->query( $sql );

	return true;
}


/**
 * Check if a string contains abusive substrings
 *
 * Note: Letting the database do the LIKE %% match is a little faster than doing
 * it in PHP, not to mention the incredibly long overhead of preloading the list
 * into PHP.
 *
 * @param string The thing you are looking for keywords in (a comment, a URL, etc)
 * @param string String for counter to increment (comments, referers, messages),
 * or false if we do not want/need increment any counter
 * @return string|boolean blacklisted keyword found or false if no spam detected
 */
function antispam_check( $haystack, $store_as = false )
{
	global $DB;
	global $localtimenow;
	global $Timer;

	$Timer->resume( 'antispam_url' ); // resuming to get the total number

	if( ! $store_as )
	{
		// we are not storing a counter increment, get a single var
		$block = $DB->get_var( 'SELECT aspm_string
			FROM T_antispam
			WHERE '.$DB->quote( $haystack ).'
			LIKE CONCAT("%",aspm_string,"%")
			LIMIT 0, 1', 0, 0, 'Check URL against antispam blacklist (var)' );
	}
	else
	{
		// we need the counters so get the row and work out $block
		$block = false;
		if( $row = $DB->get_row( 'SELECT *
			FROM T_antispam
			WHERE '.$DB->quote( $haystack ).'
			LIKE CONCAT("%",aspm_string,"%")
			LIMIT 0, 1', 0, 0, 'Check URL against antispam blacklist (row)', ARRAY_A ) )
		{
			$block = $row->aspm_string;
		}
	}

	if( $block )
	{
		if( $store_as ) // comments, referers, messages
		{
			// update the antispam table with an increase in the counter and lastseen date
			$store_as = 'aspm_'.$store_as;
			$counter = $row->$store_as + 1;

			$now = date2mysql( $localtimenow );
			$DB->query( "UPDATE T_antispam
				SET aspm_lastseen = '{$now}', $store_as = $counter
				WHERE aspm_string = '{$block}'" );
		}
		return $block; // SPAM detected!
	}
	$Timer->pause( 'antispam_url' );

	return false; // no problem.
}


/**
 * Remove an entry from the ban list
 *
 * @param integer "keyword" (domain) ID to be removed from antispam table
 */
function antispam_delete( $string_ID )
{
	global $DB;

	$sql = "DELETE FROM T_antispam WHERE aspm_ID = $string_ID";
	$DB->query( $sql );
}


/**
 * Get the base domain that could be blacklisted from an URL
 *
 * We want to concentrate on the main domain and we want to prefix it with either
 * a . or a // in order not to blacklist too large.
 *
 * @param string URL or domain
 * @return string|false the pattern to match this domain in the blacklist; false if
 * we could not extract the base domain
 */
function get_ban_domain( $url )
{
	// Remove http:// part + everything after the last path element ( '/' alone is ignored on purpose )
	$domain = preg_replace( '~^ ([a-z]+://)? ([^/#]+) (/ ([^/]*/)+ )? .* ~xi', '\\2\\3', $url );

	if( preg_match( '~^[0-9.]+$~', $domain ) )
	{
		// All numeric = IP address, don't try to cut it any further
		return '//'.$domain;
	}

	// Remove any www*. prefix
	$base_domain = preg_replace( '~^(www \w* \. )~xi', '', $domain );
	if( empty( $base_domain ) )
	{
		return false;
	}

	if( strlen( $base_domain ) < strlen( $domain ) )
	{
		// The guy is spamming with subdomains (or www)
		return '.'.$base_domain;
	}

	// The guy is spamming with the base domain
	return '//'.$base_domain;
}

?>
