<?php
/**
 * This file implements various Image File handling functions
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package files
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * @return array 'key' => 'name'
 */
function get_available_thumb_sizes()
{
	global $thumbnail_sizes;

	$thumb_size_names = array();

	foreach( $thumbnail_sizes as $key=>$dummy )
	{
		$thumb_size_names[$key] = $key;
	}

	return $thumb_size_names;
}


/**
 * Crop dimensions to fit into a constrained size, while preserving aspect ratio
 *
 * @param integer source width
 * @param integer source height
 * @param integer constrained width
 * @param integer constrained height
 */
function crop_to_constraint( $src_width, $src_height, $max_width, $max_height )
{
	$src_ratio = $src_width / $src_height;
	if( $max_width / $max_height <= $src_ratio )
	{
		$x = ( $src_width - $src_height ) / 2;
		$y = 0;
		$src_width = $src_height;
	}
	else
	{
		$x = 0;
		$y = ( $src_height - $src_width ) / 2;
		$src_height = $src_width;
	}
	return array( $x, $y, $src_width, $src_height );
}


/**
 * Scale dimensions to fit into a constrained size, while preserving aspect ratio
 *
 * @param integer source width
 * @param integer source height
 * @param integer constrained width
 * @param integer constrained height
 * @return array width, height
 */
function scale_to_constraint( $src_width, $src_height, $max_width, $max_height )
{
	$src_ratio = $src_width / $src_height;
	if( $max_width / $max_height <= $src_ratio )
	{
		$width = $max_width;
		$height = (int)round( $max_width / $src_ratio );
	}
	else
	{
		$width = (int)round( $max_height * $src_ratio );
		$height = $max_height;
	}
	return array( $width, $height );
}


/**
 * Scale dimensions to fit into a constrained size, while preserving aspect ratio
 *
 * The scaling only happens if the source is larger than the constraint.
 *
 * @uses scale_to_constraint()
 * @param integer source width
 * @param integer source height
 * @param integer constrained width
 * @param integer constrained height
 * @return array width, height
 */
function fit_into_constraint( $src_width, $src_height, $max_width, $max_height )
{
	if( $src_width > $max_width || $src_height > $max_height )
	{
		return scale_to_constraint( $src_width, $src_height, $max_width, $max_height );
	}

	return array( $src_width, $src_height );
}


/**
 * Load an image from a file into memory
 *
 * @todo (0000) imagecreatefrom*() can fail if $path is not a valid jpg/gif/png; needs handling
 * @param string pathname of image file
 * @param string mimetype of the image file
 * @return array NULL or error message, resource image handle or NULL
 */
function load_image( $path, $mimetype )
{
	// yabs> GD library uses shedloads of memory
	// @ini_set( 'memory_limit', '256M' ); // artificially inflate memory if we can
	// fp> 256M is way too high to sneak this in here. There should be some checks
	// in the systems page to warn against low memory conditions. Also i'm not sure
	// it makes sense to bump memory just for images. If you allow memory you might
	// as well allow it for anything. Anyways, this is too much to be snuck in.
	$err = NULL;
	$imh = NULL;

	switch( $mimetype )
	{
		case 'image/jpeg':
		$imh = imagecreatefromjpeg( $path );
		break;

		case 'image/gif':
		$imh = imagecreatefromgif( $path );
		break;

		case 'image/png':
		$imh = imagecreatefrompng( $path );
		break;

		default:
		// Unrecognized mime type
		$err = '!Unsupported format '.$mimetype.' (load_image())';
		break;
	}
	return array( $err, $imh );
}


/**
 * Output an image from memory to web client
 *
 * @param resource image handle
 * @param string pathname of image file
 * @param string
 * @param integer
 * @param string permissions
 * @return string
 */
function save_image( $imh, $path, $mimetype, $quality = 90, $chmod = NULL )
{
	global $Settings;

	$err = NULL;

	switch( $mimetype )
	{
		case 'image/jpeg':
		imagejpeg( $imh, $path, $quality );
		break;

		case 'image/gif':
		imagegif( $imh, $path );
		break;

		case 'image/png':
		imagepng( $imh, $path );
		break;

		default:
		// Unrecognized mime type
		$err = '!Unsupported format '.$mimetype.' (save_image)';
		break;
	}

	if( empty( $err ) )
	{
		// Make sure the file has the default permissions we want:
		if( $chmod === NULL )
		{
			$chmod = $Settings->get( 'fm_default_chmod_file' );
		}
		chmod( $path, octdec( $chmod ) );
	}
	return $err;
}


/**
 * Output an image from memory to web client
 *
 * @param resource image handle
 * @param string
 * @return string
 */
function output_image( $imh, $mimetype )
{
	$err = NULL;

	switch( $mimetype )
	{
		case 'image/jpeg':
		header('Content-type: '.$mimetype );
		imagejpeg( $imh );
		break;

		case 'image/gif':
		header('Content-type: '.$mimetype );
		imagegif( $imh );
		break;

		default:
		// Unrecognized mime type
		$err = 'Emime'; // Sort error code
		break;
	}
	return $err;
}


/**
 * Generate a thumbnail
 *
 * @uses crop_to_constraint()
 * @uses scale_to_constraint()
 * @param resource Image handle
 * @param string Type of thumbnail (fit or crop)
 * @param integer Width of thumbnail
 * @param integer Height of thumbnail
 * @return array short error code + dest image handler
 */
function generate_thumb( $src_imh, $thumb_type, $thumb_width, $thumb_height )
{
	$src_width = imagesx( $src_imh );
	$src_height = imagesy( $src_imh );

	if( $src_width <= $thumb_width && $src_height <= $thumb_height )
	{
		// There is no need to resample, use original
		return array( NULL, $src_imh );
	}

	switch( $thumb_type )
	{
		case 'crop':
		list( $src_x, $src_y, $src_width, $src_height) = crop_to_constraint( $src_width, $src_height, $thumb_width, $thumb_height );
		$dest_width = $thumb_width;
		$dest_height = $thumb_height;
		break;

		case 'fit':
		default:
		list( $dest_width, $dest_height ) = scale_to_constraint( $src_width, $src_height, $thumb_width, $thumb_height );
		$src_x = $src_y = 0;
	}

	$dest_imh = imagecreatetruecolor( $dest_width, $dest_height ); // Create a black image

	if( ! imagecopyresampled( $dest_imh, $src_imh, 0, 0, $src_x, $src_y, $dest_width, $dest_height, $src_width, $src_height ) )
	{
		return array( '!GD-library internal error (resample)', $dest_imh );
	}

	// @todo (0000) imageinterlace();

	return array( NULL, $dest_imh );
}

?>
