<?php
/**
 * This file implements the Session base class
 *
 * This file holds the {@link session_unserialize_callback()} function used
 * the Session class. A session can be bound to a user and provides functions
 * to store data in its context. All Hitlogs are also bound to a Session.
 *
 * A session tracks a given user (not necessarily logged in) while he's navigating
 * the site. A session also stores data for the length of the session. Sessions
 * are tracked with a cookie containing the session ID. The cookie also contains
 * a random key to prevent sessions hacking.
 *
 * @todo (1111) vars type 'foo' need useful/valid descriptions
 * @todo (1111) session_unserialize_callback() and session_unserialize_load_all_classes()
 * are leftovers from when we had "load_class()" but pretty much do nothing and
 * need to be totally pulled out. I'm too lazy to do it right now though, so it's
 * another todo to do :)
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package sessions
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * Session base class
 *
 * @package sessions
 */
class Session
{
	/**
	 * The ID of the session
	 * @var integer
	 */
	var $ID;
	/**
	 * The session key (to be used in URLs)
	 * @var string
	 */
	var $key;
	/**
	 * The user ID for the user of the session (NULL for anonymous (not logged in) user)
	 * @var integer
	 */
	var $user_ID;
	/**
	 * Is the session validated?
	 *
	 * This means that it was created from a received cookie.
	 *
	 * @var boolean
	 */
	var $is_validated = false;
	/**
	 * Data stored for the session
	 *
	 * This holds an array( expire, value ) for each data item key.
	 *
	 * @var array
	 */
	var $_data;
	/**
	 * FIX THIS
	 * @var foo
	 */
	var $_session_needs_save = false;

	/**
	 * Constructor
	 *
	 * If valid session cookie received: pull session from DB Otherwise, INSERT a
	 * session into DB
	 */
	function Session()
	{
		global $cookie_domain;
		global $cookie_expires;
		global $cookie_path;
		global $cookie_session;
		global $current_User;
		global $DB;
		global $Hit;
		global $localtimenow;
		global $Messages;
		global $no_session;
		global $Settings;

		$session_cookie = param_cookie( $cookie_session, 'string', '' );
		if( empty( $session_cookie ) )
		{
			// debuglog-add removed
		}
		else
		{
			// session ID sent by cookie
			if( ! preg_match( '~^(\d+)_(\w+)$~', $session_cookie, $match ) )
			{
				// debuglog-add removed
			}
			else
			{
				// We have a valid session cookie
				$session_id_by_cookie = $match[1];
				$session_key_by_cookie = $match[2];

				$row = $DB->get_row( 'SELECT sess_ID, sess_key, sess_data, sess_user_ID
					FROM T_sessions
					WHERE sess_ID = '.$DB->quote( $session_id_by_cookie ).'
					AND sess_key = '.$DB->quote( $session_key_by_cookie ).'
					AND UNIX_TIMESTAMP(sess_lastseen) > '.( $localtimenow - $Settings->get( 'timeout_sessions' ) ) );
				if( empty( $row ) )
				{
					// debuglog-add removed
				}
				else
				{
					// ID + key are valid: load data
					$this->ID = $row->sess_ID;
					$this->key = $row->sess_key;
					$this->user_ID = $row->sess_user_ID;
					$this->is_validated = true;

					if( empty( $row->sess_data ) )
					{
						$this->_data = array();
					}
					else
					{
						// Some session data has been previsouly stored
						// Unserialize session data (using an own callback that should provide class definitions)
						$old_callback = ini_set( 'unserialize_callback_func', 'session_unserialize_callback' );
						if( $old_callback === false || is_null( $old_callback ) /* disabled, reported with PHP 5.2.5 */ )
						{
							// NULL if ini_set has been disabled for security reasons
							// Brutally load all classes that we might need:
							session_unserialize_load_all_classes();
						}
						// @todo (0000) dh> This can fail, if there are special chars in sess_data:
						// It will be encoded in $app_charset _after_ "SET NAMES", but get retrieved
						// here, _before_ any "SET NAMES" (if $app_db_config['connection_charset']
						// is not set (default))!
						$this->_data = @unserialize( $row->sess_data );

						if( $old_callback !== false )
						{
							// Restore the old callback if we changed it
							ini_set( 'unserialize_callback_func', $old_callback );
						}

						if( ! is_array( $this->_data ) )
						{
							$this->_data = array();
						}
						else
						{
							// Load a Messages object from session data, if available
							if( ( $sess_Messages = $this->get( 'Messages' ) ) && is_a( $sess_Messages, 'log' ) )
							{
								// @todo (0000) dh> "old" messages should rather get prepended to
								// any existing ones from the current request, rather than appended
								$Messages->add_messages( $sess_Messages->messages );
								$this->delete( 'Messages' );
							}
						}
					}
				}
			}
		}

		if( $this->ID )
		{
			// there was a valid session before; data needs to be updated at page exit (lastseen)
			$this->_session_needs_save = true;
		}
		elseif( empty( $no_session ) )
		{
			// create a new session! 
			$this->key = generate_random_key(32);

			// We need to INSERT now because we need an ID now! (for the cookie)
			$DB->query( "
				INSERT INTO T_sessions( sess_key, sess_lastseen, sess_ipaddress )
				VALUES (
					'".$this->key."',
					'".date( 'Y-m-d H:i:s', $localtimenow )."',
					".$DB->quote( $Hit->IP )."
				)" );

			$this->ID = $DB->insert_id;

			// Set a cookie valid for ~ 10 years:
			setcookie( $cookie_session, $this->ID.'_'.$this->key, time() + 315360000, $cookie_path, $cookie_domain );
		}
	}


	/**
	 * Attach a User object to the session
	 *
	 * @param User The user to attach
	 */
	function set_User( $User )
	{
		return $this->set_user_ID( $User->ID );
	}


	/**
	 * Attach a user ID to the session
	 *
	 * NOTE: ID gets saved to DB on shutdown. This may be a "problem" when querying
	 * T_sessions for sess_user_ID.
	 *
	 * @param integer The ID of the user to attach
	 */
	function set_user_ID( $user_ID )
	{
		if( $user_ID != $this->user_ID )
		{
			global $DB;
			global $UserSettings;

			if( ! $UserSettings->get( 'login_multiple_sessions', $user_ID ) )
			{
				// The user does not want to have multiple sessions open at the same time
				// Invalidate previous sessions:
				$DB->query( 'UPDATE T_sessions SET sess_key = NULL WHERE sess_user_ID = '.$DB->quote( $user_ID ).' AND sess_ID != '.$this->ID );
			}

			$this->user_ID = $user_ID;
			$this->_session_needs_save = true;
		}
	}


	/**
	 * Logout the user, by invalidating the session key and unsetting {@link $user_ID}.
	 *
	 * We want to keep the user in the session log, but we're unsetting {@link $user_ID},
	 * which refers to the current session. Because the session key is invalid/broken,
	 * on the next request a new session will be started.
	 *
	 * NOTE: we MIGHT want to link subsequent sessions together if we want to keep track...
	 */
	function logout()
	{
		global $cookie_domain;
		global $cookie_path;
		global $cookie_session;

		// Invalidate the session key (no one will be able to use this session again)
		$this->key = NULL;
		// We don't need to keep old data
		$this->_data = array();
		$this->_session_needs_save = true;
		$this->dbsave();

		// Unset user_ID after invalidating/saving the session above, to keep the user
		// info attached to the old session.
		$this->user_ID = NULL;

		// clean up the session cookie
		setcookie( $cookie_session, '', 200000000, $cookie_path, $cookie_domain );
	}


	/**
	 * Check if session has a user attached
	 *
	 * @return boolean
	 */
	function has_User()
	{
		return ! empty( $this->user_ID );
	}


	/**
	 * Get the attached User
	 *
	 * @return false|User
	 */
	function & get_User()
	{
		if( ! empty( $this->user_ID ) )
		{
			$UserCache = & get_Cache( 'UserCache' );
			return $UserCache->get_by_ID( $this->user_ID );
		}

		$r = false;
		return $r;
	}


	/**
	 * Get a data value for the session
	 *
	 * This checks for the data to be expired and unsets it then.
	 *
	 * @param string Name of the data's key.
	 * @param mixed Default value to use if key is not set or has expired. (since 1.10.0)
	 * @return mixed The value, if set; otherwise $default
	 */
	function get( $param, $default = NULL )
	{
		global $localtimenow;

		if( isset( $this->_data[$param] ) )
		{
			if( array_key_exists( 1, $this->_data[$param] ) // can be NULL!
				&& ( is_null( $this->_data[$param][0] ) || $this->_data[$param][0] > $localtimenow ) ) // check for expired data
			{
				return $this->_data[$param][1];
			}
			else
			{
				// expired or old format (without 'value' key)
				unset( $this->_data[$param] );
				$this->_session_needs_save = true;
			}
		}

		return $default;
	}


	/**
	 * Set a data value for the session
	 *
	 * Updated values get saved to the DB automatically on shutdown, in {@link shutdown()}.
	 *
	 * @param string Name of the data's key.
	 * @param mixed The value
	 * @param integer Time in seconds for data to expire (0 to disable).
	 */
	function set( $param, $value, $expire = 0 )
	{
		global $localtimenow;

		if( ! isset( $this->_data[$param] )
		 || ! is_array( $this->_data[$param] ) // deprecated: check to transform 1.6 session data to 1.7
		 || $this->_data[$param][1] != $value
		 || $expire != 0 )
		{
			// There is something to update
			$this->_data[$param] = array( ( $expire ? ( $localtimenow + $expire ) : NULL ), $value );

			if( $param == 'Messages' )
			{
				// also set boolean to not call CachePageContent plugin event on next request
				$this->set( 'core.no_CachePageContent', 1 );
			}

			$this->_session_needs_save = true;
		}
	}


	/**
	 * Delete a value from the session data
	 *
	 * @param string Name of the data's key.
	 */
	function delete( $param )
	{
		if( isset( $this->_data[$param] ) )
		{
			unset( $this->_data[$param] );
			$this->_session_needs_save = true;
		}
	}


	/**
	 * Updates session data in database
	 *
	 * NOTE: Debuglog additions will may not be displayed since the debuglog may already
	 * have been displayed (shutdown function)
	 */
	function dbsave()
	{
		global $DB;
		global $Hit;
		global $localtimenow;
		global $no_session;

		if( ! $this->_session_needs_save )
		{
			// There have been no changes since the last save
			return false;
		}

		if( ! empty( $no_session ) )
		{
			return false;
		}

		$sess_data = empty( $this->_data ) ? NULL : serialize( $this->_data );

		// Note: The key actually only needs to be updated on a logout.
		// Note: we increase the hitcoutn every time. That assumes that there will be no 2
		// calls for a single hit. Anyway it is not a big problem if this number is approximate.
		$sql = "UPDATE T_sessions SET
			sess_hitcount = sess_hitcount + 1,
			sess_lastseen = '".date( 'Y-m-d H:i:s', $localtimenow )."',
			sess_data = ".$DB->quote( $sess_data ).",
			sess_ipaddress = '".$Hit->IP."',
			sess_key = ".$DB->quote( $this->key );
		if( ! is_null( $this->user_ID ) )
		{
			// We do NOT erase existing IDs at logout. We only want to set IDs at login
				$sql .= ", sess_user_ID = ".$this->user_ID;
		}
		$sql .= " WHERE sess_ID = ".$this->ID;

		$DB->query( $sql, 'Session::dbsave()' );

		$this->_session_needs_save = false;
	}


	/**
	 * Reload session data
	 *
	 * This is needed if the running process waits for a child process to write data
	 * into the Session, e.g. the captcha plugin in test mode waiting for the Debuglog
	 * output from the process that created the image (included through an IMG tag).
	 */
	function reload_data()
	{
		global $DB;

		if( empty( $this->ID ) )
		{
			return false;
		}

		$sess_data = $DB->get_var( '
			SELECT sess_data FROM T_sessions
			WHERE sess_ID = '.$this->ID );

		$sess_data = @unserialize( $sess_data );
		if( $sess_data === false )
		{
			$this->_data = array();
		}
		else
		{
			$this->_data = $sess_data;
		}
	}

}


/**
 * This gets used as a {@link unserialize()} callback function
 *
 * This is responsible for loading the requested class.
 *
 * IMPORTANT: when modifying this also modify {@link session_unserialize_load_all_classes()}
 *
 * @return boolean True, if the required class could be loaded; false, if not
 */
function session_unserialize_callback( $classname )
{
	switch( strtolower( $classname ) )
	{
		case 'blog':
		case 'blogsettings':
		case 'comment':
		case 'item':
		case 'group':
		case 'user':
		return true;
	}
	return false;
}


/**
 * When session_unserialize_callback() cannot be registered to do some smart loading,
 * then we fall back to this function and load everything with brute force...
 */
function session_unserialize_load_all_classes()
{
	// this used to do a handful of "load_class()" bits that returned true no matter
	// what haven't yanked this function due to it is used above and I'm lazy ;)
}

?>
