<?php
/**
 * This file implements the UserSettings class
 *
 * This class handles user_ID/name/value triplets.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package users
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * UserSettings class
 *
 * @package users
 */
class UserSettings extends AbstractSettings
{
	/**
	 * The default settings to use, when a setting is not given in the database
	 * @var array
	 */
	var $_defaults = array(
		'display_icon_legend' => 0,
		'control_form_abortions' => 1,
		'focus_on_first_input' => 0,
		'pref_browse_tab' => 'full',
		'pref_edit_tab' => 'expert', // was 'simple' but simple is going away :)
		'fm_imglistpreview' => 1,
		'fm_showdate' => 'compact',
		'fm_allowfiltering' => 'simple',
		'blogperms_layout' => 'default', // selected view in blog (user/group) perms
		'login_multiple_sessions' => 1, // allow multiple concurrent sessions? (PARAMOUNT ON DEMO SERVER)
		'results_per_page' => 50,
		'num_admin_blogs' => 10,
	);

	/**
	 * Constructor
	 */
	function UserSettings()
	{
		parent::AbstractSettings( 'T_user_settings', array( 'uset_user_ID', 'uset_name' ), 'uset_value', 1 );
	}


	/**
	 * Get a setting from the DB user settings table
	 *
	 * @param string name of setting
	 * @param integer User ID (by default $current_User->ID will be used)
	 */
	function get( $setting, $user_ID = NULL )
	{
		if( ! isset( $user_ID ) )
		{
			global $current_User;

			if( ! isset( $current_User ) )
			{
				// no current/logged in user
				return $this->get_default( $setting );
			}

			$user_ID = $current_User->ID;
		}

		return parent::get( $user_ID, $setting );
	}


	/**
	 * Temporarily sets a user setting ({@link dbupdate()} writes it to DB)
	 *
	 * @param string name of setting
	 * @param mixed new value
	 * @param integer User ID (by default $current_User->ID will be used)
	 */
	function set( $setting, $value, $user_ID = NULL )
	{
		if( ! isset( $user_ID ) )
		{
			global $current_User;

			if( ! isset( $current_User ) )
			{
				// no current/logged in user
				return false;
			}

			$user_ID = $current_User->ID;
		}

		return parent::set( $user_ID, $setting, $value );
	}


	/**
	 * Mark a setting for deletion ({@link dbupdate()} writes it to DB)
	 *
	 * @param string name of setting
	 * @param integer User ID (by default $current_User->ID will be used)
	 */
	function delete( $setting, $user_ID = NULL )
	{
		if( ! isset( $user_ID ) )
		{
			global $current_User;

			if( ! isset( $current_User ) )
			{
				// no current/logged in user
				return false;
			}

			$user_ID = $current_User->ID;
		}

		return parent::delete( $user_ID, $setting );
	}


	/**
	 * Get a param from Request and save it to UserSettings, or default to previously saved user setting
	 *
	 * If the user setting was not set before (and there's no default given that gets
	 * returned), $default gets used.
	 *
	 * @todo (0000) Move this to _abstractsettings.class.php - the other Settings
	 * object can also make use of it!
	 * @param string Request param name
	 * @param string User setting name. Make sure this is unique!
	 * @param string Force value type to one of:
	 * - integer
	 * - float
	 * - string (strips (HTML-)Tags, trims whitespace)
	 * - array
	 * - object
	 * - null
	 * - html (does nothing)
	 * - '' (does nothing)
	 * - '/^...$/' check regexp pattern match (string)
	 * - boolean (will force type to boolean, but you can't use 'true' as a default
	 *   since it has special meaning. There is no real reason to pass booleans on a
	 *   URL though. Passing 0 and 1 as integers seems to be best practice).
	 *   Value type will be forced only if resulting value (probably from default then) is !== NULL
	 * @param mixed Default value or TRUE if user input required
	 * @param boolean Do we need to memorize this to regenerate the URL for this page?
	 * @param boolean Override if variable already set
	 * @return NULL|mixed NULL, if neither a param was given nor {@link $UserSettings} knows about it.
	 */
	function param_Request( $param_name, $uset_name, $type = '', $default = '', $memorize = false, $override = false ) // we do not force setting it..
	{
		$value = param( $param_name, $type, NULL, $memorize, $override, false ); // we pass NULL here, to see if it got set at all

		if( $value !== false )
		{
			// we got a value
			$this->set( $uset_name, $value );
			$this->dbupdate();
		}
		else
		{
			// get the value from user settings
			$value = $this->get( $uset_name );

			if( is_null( $value ) )
			{
				// it's not saved yet and there's not default defined ( $_defaults )
				$value = $default;
			}
		}

		set_param( $param_name, $value );
		return get_param( $param_name );
	}
}

?>
