<?php
/**
 * This file implements the "Universal Item List" widget
 *
 * To call this widget directly in your template (bypassing containers), use
 * the following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'item_list',
 * 	'widget_title' => 'Items', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 12 params that you can set when with a direct widget call:<ul>
 * <li>'title_link' => default is false, which means the block title is not linked
 * to the blog.</li>
 * <li>'item_type' => default is '#', which means the default item type (depends on
 * your installation configuration). Options are '' for all item types, and each
 * item type you have in your installation. Your configuration may result in
 * additional options!!!</li>
 * <li>'thumb_size' => default is 'crop-80x80', see {@link $thumbnail_sizes}</li>
 * <li>'follow_mainlist' => default is 'none', which means you will not restrict the
 * list to contents related to what is displayed in the main area. Optionally
 * you can use 'tags' to restrict the list to items related by tags from the
 * mainlist.</li>
 * <li>'blog_ID' => default is empty/null which will give you an item list from the
 * current blog. Use a blog ID (number) for items from a specific blog.</li>
 * <li>'item_group_by' => default is 'none' which means no grouping. Optionally you
 * can set this to 'category' for grouping by category.</li>
 * <li>'order_by' => default is 'datestart' which orders your items by date. See
 * {@link get_available_sort_options()} for available options.</li>
 * <li>'order_dir' => default is 'DESC' which means descending order. Optionally you
 * can set this to 'ASC' for ascending order.</li>
 * <li>'limit' => default is 20, which means no more than 20 items will be shown.</li>
 * <li>'disp_title' => default is true, which will give you titles displayed ... which
 * is pretty stupid because what else would you display in a freaking item list!</li>
 * <li>'item_title_link_type' => default is 'auto' which means "link to URL if there
 * is one, else permalink". Optionally you can use 'permalink' (Item permalink),
 * 'linkto_url' (Item URL), or, 'none' (Nowhere)</li>
 * <li>'attached_pics' => default is 'none' for "None", options are 'first' for
 * "Display first", and 'all' for "Display all".</li>
 * <li>'item_pic_link_type' default is 'auto' for "Automatic", options are 'permalink'
 * for "Item permalink", 'linkto_url' for "Item URL", and 'none'for "Nowhere".</li>
 * <li>'disp_excerpt' => default is false, change to true if you want to display the
 * excerpt from each item.</li>
 * <li>'disp_teaser' => default is false, change to true if you want to display a
 * content teaser for each item.</li>
 * <li>'disp_teaser_maxwords' => default is 20, this is the maximum number of words
 * a content teaser will contain.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * item_list_Widget class
 *
 * @package widgets
 */
class item_list_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function item_list_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'item_list' );
		if( empty( $this->widget_name ) && empty( $this->widget_title ) )
		{
			// this widget gets extended by others :-S
			$this->widget_name = T_('Universal Item List');
			$this->widget_title = T_('Items');
		}
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;
		global $BlogCache;
		global $MainList;
		global $timestamp_max;
		global $timestamp_min;

		$this->init_display( $params ); // just in case it hasn't been done before

		$listBlog = ( $this->disp_params['blog_ID'] ? $BlogCache->get_by_ID( $this->disp_params['blog_ID'], false ) : $Blog );

		if( empty( $listBlog ) )
		{
			echo $this->disp_params['block_start'];
			echo T_('The requested Blog doesn\'t exist any more!');
			echo $this->disp_params['block_end'];
			return;
		}

		// Create ItemList ... Note: we pass a widget specific prefix in
		// order to make sure to never interfere with the mainlist
		$limit = $this->disp_params['limit'];
		if( $this->disp_params['disp_teaser'] )
		{
			// We want to show some of the post content, we need to load more info: use ItemList2
			$ItemList = new ItemList2( $listBlog, $timestamp_min, $timestamp_max, $limit, 'ItemCache', $this->code.'_' );
		}
		else
		{
			// no excerpts, use ItemListLight
			$ItemList = new ItemListLight( $listBlog, $timestamp_min, $timestamp_max, $limit, 'ItemCacheLight', $this->code.'_' );
		}

		// Filter list
		$filters = array(
			'orderby' => $this->disp_params['order_by'],
			'order' => $this->disp_params['order_dir'],
			'unit' => 'posts', // We want to advertise all items (not just a page or a day)
		);
		if( $this->disp_params['item_type'] != '#' )
		{
			// Not "default", restrict to a specific type (or '' for all)
			$filters['types'] = $this->disp_params['item_type'];
		}

		if( $this->disp_params['follow_mainlist'] == 'tags' )
		{
			// Restrict to Item tagged with some tag used in the Mainlist
			if( ! isset( $MainList ) )
			{
				// Nothing to follow, don't display anything
				return false;
			}
			$all_tags = $MainList->get_all_tags();
			if( empty( $all_tags ) )
			{
				// Nothing to follow, don't display anything
				return false;
			}

			$filters['tags'] = implode(',',$all_tags);
		}

		$category_mode = false;
		if( $this->disp_params['item_group_by'] == 'category' )
		{
			// Group by category
			$category_mode = true;

			// This is the list of categories to restrict the linkblog to (cats will
			// be displayed recursively). Example: $linkblog_cat = '4,6,7';
			$linkblog_cat = '';

			// This is the array if categories to restrict the linkblog to (non
			// recursive). Example: $linkblog_catsel = array( 4, 6, 7 );
			$linkblog_catsel = array(); // $cat_array;

			// Compile cat array stuff
			$linkblog_cat_array = array();
			$linkblog_cat_modifier = '';

			compile_cat_array( $linkblog_cat, $linkblog_catsel, /* by ref */ $linkblog_cat_array, /* by ref */ $linkblog_cat_modifier, $listBlog->ID );

			$filters['cat_array'] = $linkblog_cat_array;
			$filters['cat_modifier'] = $linkblog_cat_modifier;
			$filters['orderby'] = 'main_cat_ID '.$filters['orderby'];
		}

		$ItemList->set_filters( $filters, false ); // we don't want to memorize these params

		// Run the query
		$ItemList->query();

		if( ! $ItemList->result_num_rows )
		{
			// Nothing to display
			return;
		}

		echo $this->disp_params['block_start'];
		$title = sprintf( ( $this->disp_params['title_link'] ? '<a href="'.$listBlog->gen_blogurl().'" rel="nofollow">%s</a>' : '%s' ), $this->disp_params['widget_title'] );
		$this->disp_title( $title );
		echo $this->disp_params['list_start'];

		if( $category_mode )
		{
			// List grouped by category
			while( $Item = & $ItemList->get_category_group() )
			{
				// Open new cat
				echo $this->disp_params['item_start'];
				$Item->main_category();
				echo $this->disp_params['group_start'];
				while( $Item = & $ItemList->get_item() )
				{
					$this->disp_contents( $Item );
				}
				// Close cat
				echo $this->disp_params['group_end'];
				echo $this->disp_params['item_end'];
			}
		}
		else
		{
			// Plain list
			while( $Item = & $ItemList->get_item() )
			{
				$this->disp_contents( $Item );
			}
		}

		echo $this->disp_params['list_end'];
		echo $this->disp_params['block_end'];
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Can list Items (Posts/Pages/Links...) in a variety of ways.');
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @todo (4334) oldcore: filter this out from all "SIMPLE" lists and keep it only in universal list
	 * 	'cat_IDs' => array(
	 * 		'label' => T_('Categories'),
	 * 		'size' => 15,
	 * 		'note' => T_('List category IDs separated by ,'),
	 * 	),
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$ItemTypeCache = & get_Cache( 'ItemTypeCache' );
		$item_type_options = array(
			'#' => T_('Default'),
			'' => T_('All'),
		) + $ItemTypeCache->get_option_array();

		$r = array_merge( array(
			'title_link' => array(
				'label' => T_('Link to blog'),
				'type' => 'checkbox',
				'defaultvalue' => false,
				'note' => T_('Link the block title to the blog?'),
			),
			'item_type' => array(
				'label' => T_('Item type'),
				'type' => 'select',
				'options' => $item_type_options,
				'defaultvalue' => '#',
				'note' => T_('What kind of items do you want to list?'),
			),
			'follow_mainlist' => array(
				'label' => T_('Follow Main List'),
				'type' => 'select', // should be a radio button set
				'options' => array(
					'no' => T_('No'),
					'tags' => T_('By tags'),
				), // may be extended
				'defaultvalue' => 'no',
				'note' => T_('Do you want to restrict to contents related to what is displayed in the main area?'),
			),
			'blog_ID' => array(
				'label' => T_('Blog'),
				'size' => 4,
				'note' => T_('ID of the blog to use, leave empty for the current blog.'),
			),
			'item_group_by' => array(
				'label' => T_('Group by'),
				'type' => 'select', // should be a radio button set
				'options' => array(
					'none' => T_('None'),
					'category' => T_('By Category'),
				),
				'defaultvalue' => 'none',
				'note' => T_('Do you want to group the Items?'),
			),
			'order_by' => array(
				'label' => T_('Order by'),
				'type' => 'select',
				'options' => get_available_sort_options(),
				'defaultvalue' => 'datestart',
				'note' => T_('How to sort the items'),
			),
			'order_dir' => array(
				'label' => T_('Direction'),
				'type' => 'select', // should be a radio button set
				'options' => array(
					'ASC' => T_('Ascending'),
					'DESC' => T_('Descending'),
				),
				'defaultvalue' => 'DESC',
				'note' => T_('How to sort the items'),
			),
			'limit' => array(
				'label' => T_('Max items'),
				'size' => 4,
				'defaultvalue' => 20,
				'note' => T_('Maximum number of items to display.'),
			),
			'item_title_link_type' => array(
				'label' => T_('Link titles'),
				'type' => 'select',
				'options' => array(
					'auto' => T_('Automatic'),
					'permalink' => T_('Item permalink'),
					'linkto_url' => T_('Item URL'),
					'none' => T_('Nowhere'),
				),
				'defaultvalue' => 'auto',
				'note' => T_('Where should titles be linked to?'),
			),
			'disp_excerpt' => array(
				'label' => T_('Excerpt'),
				'type' => 'checkbox',
				'defaultvalue' => false,
				'note' => T_('Display excerpt for each item.'),
			),
			'disp_teaser' => array(
				'label' => T_('Content teaser'),
				'type' => 'checkbox',
				'defaultvalue' => false,
				'note' => T_('Display content teaser for each item.'),
			),
			'disp_teaser_maxwords' => array(
				'label' => T_('Max Words'),
				'type' => 'integer',
				'defaultvalue' => 20,
				'note' => T_('Max number of words for the teasers.'),
			),
		), parent::get_param_definitions( $params ) );

		return $r;
	}

// function(s) unique to this widget ------------------------------------------

	/**
	 * Support function for this widget
	 *
	 * @param Item
	 * @return boolean TRUE - if content is displayed
	 */
	function disp_contents( & $Item )
	{
		echo $this->disp_params['item_start'];
		$Item->title( array(
			'link_type' => $this->disp_params['item_title_link_type'],
		) );

		if( $this->disp_params['disp_excerpt'] )
		{
			$excerpt = $Item->dget( 'excerpt', 'htmlbody' );
			if( ! empty( $excerpt ) )
			{
				// Note: Excerpts are plain text -- no html (at least for now)
				echo '<div class="item_excerpt">'.$excerpt.'</div>';
			}
		}

		if( $this->disp_params['disp_teaser'] )
		{
			// we want to show some or all of the post content
			$content = $Item->get_content_teaser( 1, false, 'htmlbody' );

			if( $words = $this->disp_params['disp_teaser_maxwords'] )
			{
				// limit number of words
				$content = strmaxwords( $content, $words,
					array(
						'continued_link' => $Item->get_permanent_url(),
						'continued_text' => '&hellip;',
					)
				);
			}
			echo '<div class="item_content">'.$content.'</div>';

			/**
			 * @todo (0000) fp> does that really make sense?
			 * we're no longer in a linkblog/linkroll use case here, are we?
			 */
			/*
			// display "more" (read more/follow up) link
			$Item->more_link( array(
				'before' => '',
				'after' => '',
				'link_text' => T_('more').' &raquo;',
			) );
			*/
		}

		echo $this->disp_params['item_end'];
	}

}

?>
