<?php
/**
 * This file implements the "Tag Cloud" widget
 *
 * To call this widget directly in your template (bypassing containers), use the
 * following code modified with any params you need to customize:
 * <code>
 * // display a widget without a container
 * template_widget( array(
 * 	'widget' => 'tag_cloud',
 * 	'widget_title' => 'Tag cloud', // default shown, customize as desired
 * ) );
 * </code>
 *
 * This widget has 6 params that you can set when with a direct widget call:<ul>
 * <li>'max_tags' => default is 50, this is the maximum number of tags in your cloud.</li>
 * <li>'tag_separator' => default is ' ', this is what goes between tags.</li>
 * <li>'tag_min_size' => default is 8, this is the smallest tag size.</li>
 * <li>'tag_max_size' => default is 22, this is the largest tag size.</li>
 * <li>'tag_cloud_start' => default is '&lt;p class="tag_cloud">', accepts html input,
 * this begins your tag cloud.</li>
 * <li>'tag_cloud_end' => default is '&lt;/p>', accepts html input, this ends your tag
 * cloud.</li></ul>
 *
 * @see ComponentWidget::init_display()
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * tag_cloud_Widget class
 *
 * @package widgets
 */
class tag_cloud_Widget extends ComponentWidget
{
	/**
	 * Constructor
	 */
	function tag_cloud_Widget( $db_row = NULL )
	{
		// Call parent constructor
		parent::ComponentWidget( $db_row, 'widget', 'tag_cloud' );
		$this->widget_name = T_('Tag Cloud');
		$this->widget_title = T_('Tag cloud');
	}


	/**
	 * Display the widget!
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;
		global $DB;

		if( empty( $Blog ) )
		{
			// Nothing to display
			return;
		}

		$this->init_display( $params );

		// get list of relevant blogs
		$sql = 'SELECT LOWER(tag_name) AS tag_name, COUNT(DISTINCT itag_itm_ID) AS tag_count
			FROM T_tags INNER JOIN T_item_tags ON itag_tag_ID = tag_ID
			INNER JOIN T_item_cats ON itag_itm_ID = postcat_post_ID
			INNER JOIN T_categories ON postcat_cat_ID = cat_ID
			INNER JOIN T_items ON itag_itm_ID = post_ID
			WHERE '.$Blog->get_sql_where_aggregate_blog_IDs('cat_blog_ID').'
			AND post_status = "published" AND post_datestart < NOW()
			GROUP BY tag_name ORDER BY tag_count DESC LIMIT '.$this->disp_params['max_tags'];
		$results = $DB->get_results( $sql, OBJECT, 'Get tags' );
		if( empty( $results ) )
		{
			// No tags!
			return;
		}

		$max_count = $results[0]->tag_count;
		$min_count = $results[count( $results )-1]->tag_count;
		$count_span = max( 1, $max_count - $min_count );
		$max_size = $this->disp_params['tag_max_size'];
		$min_size = $this->disp_params['tag_min_size'];
		$size_span = $max_size - $min_size;

		usort( $results, array( $this, 'tag_cloud_cmp' ) );

		echo $this->disp_params['block_start'];
		$this->disp_title();
		echo $this->disp_params['tag_cloud_start'];
		$count = 0;
		foreach( $results as $row )
		{
			if( $count > 0 )
			{
				echo $this->disp_params['tag_separator'];
			}
			// If there's a space in the tag name, quote it
			$tag_name_disp = strpos( $row->tag_name, ' ' )
				? '&laquo;'.format_to_output( $row->tag_name, 'htmlbody' ).'&raquo;'
				: format_to_output( $row->tag_name, 'htmlbody' );
			$size = floor( $row->tag_count * $size_span / $count_span + $min_size );

			echo $Blog->get_tag_link( $row->tag_name, $tag_name_disp, array(
				'style' => 'font-size: '.$size.'pt;',
				'title' => sprintf( T_('%d posts'), $row->tag_count )
			) );
			$count++;
		}
		echo $this->disp_params['tag_cloud_end'];
		echo $this->disp_params['block_end'];

		return true;
	}


	/**
	 * Get widget's description
	 */
	function get_desc()
	{
		return T_('Cloud of all tags; click filters blog on selected tag.');
	}


	/**
	 * Get definitions for this widget's settings
	 *
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array_merge( array(
			'max_tags' => array(
				'label' => T_('Max # of tags'),
				'type' => 'integer',
				'size' => 4,
				'defaultvalue' => 50,
			),
			'tag_separator' => array(
				'label' => T_('Tag separator'),
				'type' => 'text',
				'maxlength' => 100,
				'defaultvalue' => ' ',
			),
			'tag_min_size' => array(
				'label' => T_('Min size'),
				'type' => 'integer',
				'size' => 3,
				'defaultvalue' => 8,
			),
			'tag_max_size' => array(
				'label' => T_('Max size'),
				'type' => 'integer',
				'size' => 3,
				'defaultvalue' => 22,
			),
			'tag_cloud_start' => array(
				'label' => T_('Cloud start'),
				'type' => 'html_input',
				'size' => 28,
				'defaultvalue' => '<p class="tag_cloud">',
			),
			'tag_cloud_end' => array(
				'label' => T_('Cloud end'),
				'type' => 'html_input',
				'defaultvalue' => '</p>',
			),
		), parent::get_param_definitions( $params ) );
		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * @todo (3069) EdB: rename this to get_title()
	 */
	function get_short_desc()
	{
		return format_to_output( $this->disp_params['widget_title'] );
	}


	/**
	 * Load params
	 */
	function load_from_Request()
	{
		parent::load_from_Request();
		// SPECIAL treatments
		if( empty( $this->param_array['tag_separator'] ) )
		{
			// Default name, don't store
			$this->set( 'tag_separator', ' ' );
		}
	}


	/**
	 * support function for display()
	 */
	function tag_cloud_cmp( $a, $b )
	{
		return strcasecmp( $a->tag_name, $b->tag_name );
	}

}

?>
