<?php
/**
 * This file implements the ComponentWidget class
 *
 * A ComponentWidget is a displayable entity that can be placed into a Container
 * on a web page.
 *
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package widgets
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * ComponentWidget class
 *
 * @package widgets
 */
class ComponentWidget extends DataObject
{
	/**
	 * Widget's code
	 * @var string The unique code for this widget (32 characters max)
	 */
	var $code;
	/**
	 * @var array Array of params used during display()
	 */
	var $disp_params;
	/**
	 * Enabled or not
	 * @var boolean Indicates whether the widget is enabled
	 */
	var $enabled;
	/**
	 * Widget's order
	 * @var integer This widget's position in the container
	 */
	var $order;
	/**
	 * Widget's params
	 * @var array Params used to display this widget
	 */
	var $params;
	/**
	 * Widget's customized params
	 * @var array Array of params which have been customized for this widget instance.
	 * This is saved to the DB as a serialized string ( $params )
	 */
	var $param_array = NULL;
	/**
	 * Plugin that handles this widget
	 * @see get_Plugin()
	 * @var object Lazy instantiated, false if this widget is not handled by a plugin
	 */
	var $Plugin;
	/**
	 * Container name
	 * @var string The container name for this widget (40 characters max)
	 */
	var $sco_name;
	/**
	 * Widget's type
	 * @var string Type of the widget (enum: widget, plugin)
	 */
	var $type;
	/**
	 * Widget blog ID
	 * @var integer The blog ID for this widget
	 */
	var $widget_blog_ID;
	/**
	 * Widget's name
	 * @var string The name of this widget from either the widget or plugin, can be
	 * changed by the user, displayed in admin interface
	 */
	var $widget_name;
	/**
	 * Widget's title
	 * @var string The title of this widget from either the widget or plugin, can be
	 * changed by the user, possibly displayed to the visiting public
	 */
	var $widget_title;
	/**
	 * NO CLUE ON THIS ONE
	 * @see ComponentWidget::T_()
	 * @var array The translations keyed by locale. They get loaded through include() of _global.php
	 */
	var $_trans = array();
	/**
	 * NO CLUE ON THIS ONE
	 * @var boolean Has the global po/_global.php file (where translation for all
	 * languages can be put into) been loaded?
	 */
	var $_trans_loaded_global = false;

	/**
	 * Constructor
	 *
	 * @param object data row from db
	 */
	function ComponentWidget( $db_row = NULL, $type = 'widget', $code = NULL )
	{
		// Call parent constructor
		parent::DataObject( 'T_widgets', 'wi_', 'wi_ID' );

		if( is_null( $db_row ) )
		{
			// We are creating an object here. Using parent:: instead of $this-> in
			// order to avoid "Inactive/Uninstalled plugin"
			parent::set( 'type', $type );
			parent::set( 'code', $code );
		}
		else
		{
			// We are loading an object
			$this->ID = $db_row->wi_ID;
			$this->widget_blog_ID = $db_row->wi_widget_blog_ID;
			$this->sco_name = $db_row->wi_sco_name;
			$this->type = $db_row->wi_type;
			$this->code = $db_row->wi_code;
			$this->params = $db_row->wi_params;
			$this->order = $db_row->wi_order;
			$this->enabled = $db_row->wi_enabled;
		}
	}


	/**
	 * Get reference to Plugin handling this Widget
	 *
	 * @return Plugin
	 */
	function & get_Plugin()
	{
		global $Plugins;

		if( is_null( $this->Plugin ) )
		{
			if( $this->type != 'plugin' )
			{
				$this->Plugin = false;
			}
			else
			{
				$this->Plugin = & $Plugins->get_by_code( $this->code );
			}
		}
		return $this->Plugin;
	}


	/**
	 * Insert object into DB based on previously recorded changes
	 *
	 * @return boolean true on success
	 */
	function dbinsert()
	{
		global $DB;

		if( $this->ID != 0 )
		{
			debug_die( 'Existing object cannot be inserted!' );
		}

		$DB->begin();
		$order_max = $DB->get_var( 'SELECT MAX(wi_order) FROM T_widgets WHERE wi_widget_blog_ID = '.$this->widget_blog_ID.' AND wi_sco_name = '.$DB->quote( $this->sco_name ), 0, 0, 'Get current max order' );
		$this->set( 'order', $order_max + 1 );
		$res = parent::dbinsert();
		$DB->commit();

		return $res;
	}


	/**
	 * Display the Widget's Title
	 *
	 * A container can prevent display of titles with 'block_display_title'. This
	 * is useful for the lists in the headers.
	 *
	 * @param string title
	 */
	function disp_title( $title = NULL )
	{
		if( is_null( $title ) )
		{
			$title = $this->disp_params['widget_title'];
		}

		if( $this->disp_params['block_display_title'] && ! empty( $title ) )
		{
			echo $this->disp_params['block_title_start'];
			echo format_to_output( $title );
			echo $this->disp_params['block_title_end'];
		}
	}


	/**
	 * Display the widget
	 *
	 * Should be overriden by each widget
	 *
	 * @uses ComponentWidget::init_display()
	 * @param array MUST contain at least the basic display params
	 */
	function display( $params )
	{
		global $Blog;
		global $Plugins;
		global $rsc_url;

		$this->init_display( $params ); // just in case it hasn't been done before

		switch( $this->type )
		{
			case 'plugin':
			// Call plugin (will return false if Plugin is not enabled)
			if( $Plugins->call_by_code( $this->code, $this->disp_params ) )
			{
				return true;
			}
			// Plugin failed (happens when a plugin has been disabled for example)
			return false;
		}
		echo "Widget $this->type : $this->code did not provide a display() method! ";
		return false;
	}


	/**
	 * Get widget's short description
	 *
	 * Should be overriden by each widget
	 */
	function get_desc()
	{
		if( $this->type == 'plugin' )
		{
			// Make sure Plugin is loaded
			if( $this->get_Plugin() )
			{
				return $this->Plugin->short_desc;
			}
			return T_('Inactive / Uninstalled plugin');
		}

		return T_('Unknown');
	}


	/**
	 * Get a clean description to display in the widget list
	 * @return string
	 */
	function get_desc_for_list()
	{
		$name = $this->get_name();

		if( $this->type == 'plugin' )
		{
			return '<strong>'.$name.'</strong> ('.T_('Plugin').')';
		}

		$short_desc = $this->get_short_desc();

		if( $name == $short_desc || empty( $short_desc ) )
		{
			return '<strong>'.$name.'</strong>';
		}

		return '<strong>'.$name.'</strong> ('.$short_desc.')';
	}


	/**
	 * Get name of widget
	 *
	 * Should be overriden by core widgets
	 */
	function get_name()
	{
		if( $this->widget_name )
		{
			// we have a name
			$name = $this->widget_name;
		}
		else
		{
			// we don't have a name
			if( $this->type == 'plugin' )
			{
				// Make sure Plugin is loaded
				if( $this->get_Plugin() )
				{
					$name = $this->Plugin->name;
				}
				else
				{
					$name = T_('Inactive / Uninstalled plugin');
				}
			}
			else
			{
				// it's a widget
				$name = T_('Unknown widget');
			}
		}

		if( $this->disp_params['widget_name'] && $name != $this->disp_params['widget_name'] )
		{
			// user has renamed us
			$name .= ' : '.$this->disp_params['widget_name'];
		}

		return $name;
	}


	/**
 	 * Get a param value
	 *
 	 * @param string
 	 * @return mixed
	 */
	function get_param( $parname )
	{
		$this->load_param_array();
		if( isset( $this->param_array[$parname] ) )
		{
			// We have a value for this param
			return $this->param_array[$parname];
		}

		// Try default values. Note we set 'infinite_loop' param to avoid calling
		// the get_param() from the get_param_definitions() function recursively
		$params = $this->get_param_definitions( NULL );
		if( isset( $params[$parname]['defaultvalue'] ) )
		{
			// We have a default value
			return $params[$parname]['defaultvalue'] ;
		}

		return NULL;
	}


	/**
	 * Get definitions for editable params
	 *
	 * @todo (1111) move widget_title to each widget, for widgets that don't have a title.
	 * @see Plugin::GetDefaultSettings()
	 * @param local params like 'for_editing' => true
	 */
	function get_param_definitions( $params )
	{
		$r = array(
			'widget_name' => array(
				'label' => T_('Name'),
				'size' => 20,
				'defaultvalue' => $this->widget_name,
				'note' => T_('Displayed in the widget list.'),
			),
			'widget_title' => array(
				'label' => T_('Title'),
				'size' => 20,
				'defaultvalue' => $this->widget_title,
				'note' => T_('Displayed in your template.'),
			),
			'widget_ID' => array(
				'label' => '<span class="dimmed">'.T_('DOM ID').'</span>',
				'size' => 20,
				'note' => T_('Replaces $wi_ID$ in your templates containers.'),
			),
			'widget_css_class' => array(
				'label' => '<span class="dimmed">'.T_('CSS Class').'</span>',
				'size' => 20,
				'note' => T_('Replaces $wi_class$ in your templates containers.'),
			),
		);

		if( $this->type == 'plugin' )
		{
			// Make sure Plugin is loaded
			if( $this->get_Plugin() )
			{
				$r = array_merge( $r, $this->Plugin->GetDefaultWidgetSettings( $params ) );
			}
		}

		return $r;
	}


	/**
	 * Get widget's title
	 *
	 * Should be overriden by each widget
	 * @todo (3069) EdB> rename this to get_title()
	 */
	function get_short_desc()
	{
		return $this->get_name();
	}


	/**
	 * Prepare display params
	 *
	 * Merge basic defaults < widget defaults < container params < DB params
	 *
	 * When called with template_widget it falls back to: basic
	 * defaults < widget defaults < calltime params < array()
	 *
	 * <code>
	 * $params = array_merge( array(
	 *   // block-level params...
	 *   'block_start' => '<div class="$wi_class$">',
	 *   'block_end' => '</div>',
	 *   'block_display_title' => true,
	 *   'block_title_start' => '<h3>',
	 *   'block_title_end' => '</h3>',
	 *   // is this maybe sub-block params?
	 *   'coll_start' => '<h4>',
	 *   'coll_end' => '</h4>',
	 *   'collist_start' => '',
	 *   'collist_end' => '',
	 *   // group and list params...
	 *   'group_start' => '<ul>',
	 *   'group_end' => '</ul>',
	 *   'list_start' => '<ul>',
	 *   'list_end' => '</ul>',
	 *   // way too many item params...
	 *   'item_start' => '<li>',
	 *   'item_end' => '</li>',
	 *   'item_text_start' => '',
	 *   'item_text' => '%s',
	 *   'item_text_end' => '',
	 *   'item_selected_start' => '<li class="selected">',
	 *   'item_selected_text' => '%s',
	 *   'item_selected_end' => '</li>',
	 *   'item_selected_text_start' => '',
	 *   'item_selected_text_end' => '',
	 *   // link params...
	 *   'link_default_class' => 'default',
	 *   'link_selected_class' => 'selected',
	 *   'link_type' => 'canonic', // 'canonic' | 'context' (context will regenerate URL injecting/replacing a single filter)
	 *   // notes params...
	 *   'notes_start' => '<div class="notes">',
	 *   'notes_end' => '</div>',
	 *   // thumb_size might be only with grid params but we need more image defaults
	 *   'thumb_size' => 'crop-80x80',
	 *   // this is probably a tag cloud param disguised as a real one
	 *   'limit' => 100,
	 * ), $widget_defaults, $params, $this->param_array );
	 * </code>
	 *
	 * @param array MUST contain at least the basic display params
	 */
	function init_display( $params )
	{
		global $admin_url;

		// generate widget defaults array
		$widget_defaults = array();
		$defs = $this->get_param_definitions( array() );
		foreach( $defs as $parname => $parmeta )
		{
			if( isset( $parmeta['defaultvalue'] ) )
			{
				$widget_defaults[$parname] = $parmeta['defaultvalue'];
			}
			else
			{
				$widget_defaults[$parname] = NULL;
			}
		}

		// Load DB configuration
		$this->load_param_array();

		$params = array_merge( array(
			// block-level params ...
			'block_start' => '<div class="$wi_class$">',
			'block_end' => '</div>',
			'block_display_title' => true,
			'block_title_start' => '<h3>',
			'block_title_end' => '</h3>',
			// is this maybe sub-block params?
			'coll_start' => '<h4>',
			'coll_end' => '</h4>',
			'collist_start' => '',
			'collist_end' => '',
			// group and list params ...
			'group_start' => '<ul>',
			'group_end' => '</ul>',
			'list_start' => '<ul>',
			'list_end' => '</ul>',
			// way too many item params ...
			'item_start' => '<li>',
			'item_end' => '</li>',
			'item_text_start' => '',
			'item_text' => '%s',
			'item_text_end' => '',
			'item_selected_start' => '<li class="selected">',
			'item_selected_text_start' => '',
			'item_selected_text' => '%s',
			'item_selected_text_end' => '',
			'item_selected_end' => '</li>',
			// link params ...
			'link_default_class' => 'default',
			'link_selected_class' => 'selected',
			'link_type' => 'canonic', // 'canonic' | 'context' (context will regenerate URL injecting/replacing a single filter)
			// notes params ...
			'notes_start' => '<div class="notes">',
			'notes_end' => '</div>',
			// thumb size might be only with grid params but we need more image defaults
			'thumb_size' => 'crop-80x80',
			// this is probably a tag cloud param disguised as a real one
			'limit' => 100,
		), $widget_defaults, $params, $this->param_array );

		// Customize params to the current widget (ID and class if desired)
		$widget_ID = ( ! empty( $params['widget_ID'] ) ? $params['widget_ID'] : $this->type.'_'.$this->code.'_'.$this->ID );
		$widget_css_class = $this->type.'_'.$this->code.( empty( $params['widget_css_class'] ) ? '' : ' '.$params['widget_css_class'] );
		$this->disp_params = str_replace( array( '$wi_ID$', '$wi_class$' ), array( $widget_ID, $widget_css_class ), $params );
	}


	/**
	 * Load params
	 */
	function load_from_Request()
	{
		load_funcs( 'plugins/_plugin.funcs.php' );

		// Loop through all widget params
		foreach( $this->get_param_definitions( array( 'for_editing' => true ) ) as $parname => $parmeta )
		{
			autoform_set_param_from_request( $parname, $parmeta, $this, 'Widget' );
		}
	}


	/**
	 * Load param array
	 */
	function load_param_array()
	{
		if( is_null( $this->param_array ) )
		{
			// Param array has not been loaded yet
			$this->param_array = @unserialize( $this->params );

			if( empty( $this->param_array ) )
			{
				// No saved param values were found
				$this->param_array = array();
			}
		}
	}


	/**
	 * Set param value
	 *
	 * @param string parameter name
	 * @param mixed parameter value
	 * @return boolean true, if a value has been set, false if it has not changed
	 */
	function set( $parname, $parvalue )
	{
		$params = $this->get_param_definitions( NULL );

		if( isset( $params[$parname] ) )
		{
			// This is a widget specific param
			$this->param_array[$parname] = $parvalue;
			// This is what'll be saved to the DB
			$this->set_param( 'params', 'string', serialize( $this->param_array ) );
			return;
		}

		switch( $parname )
		{
			default:
			return parent::set_param( $parname, 'string', $parvalue );
		}
	}


	/**
	 * Translate and escape single quotes
	 *
	 * This is to be used mainly for Javascript strings.
	 *
	 * @uses ComponentWidget::T_()
	 * @param string String to translate
	 * @param string Locale to use
	 * @return string The translated and escaped string.
	 */
	function TS_( $string, $req_locale = '' )
	{
		return str_replace( "'", "\\'", $this->T_( $string, $req_locale ) );
	}

}

?>
