<?php
/**
 * This file implements the Whosonline plugin
 *
 * @author {@link http://wonderwinds.com/ Ed Bennett}
 * @author {@link http://daniel.hahler.de/ Daniel HAHLER}
 * @author {@link http://fplanque.net/ Francois PLANQUE}
 * @author {@link http://www.jeffbearer.com/ Jeff BEARER}
 * @copyright (c) 2009 by {@link http://quamplures.net/ the Quam Plures project}
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @package plugins
 * @subpackage WhosOnline
 */
if(!defined('QP_MAIN_INIT')) die('fail');

/**
 * @package plugins
 *
 * @subpackage WhosOnline
 */
class whosonline_plugin extends Plugin
{
	var $code = 'whosonline';
	var $group = 'widgets';
	var $number_of_installs = 1;
	var $priority = 96;
	var $version = '1.0';

	/**
	 * @see Plugin::PluginInit()
	 */
	function PluginInit( & $params )
	{
		$this->name = $this->T_('Who\'s Online Widget');
		$this->short_desc = $this->T_('Displays a list of whos is online.');
		$this->long_desc = $this->T_('This plugin creates a widget that shows which logged in members and how many guest visitors have requested a page in the last 5 minutes.');
	}


  /**
	 * @see Plugin::GetDefaultWidgetSettings()
	 */
	function GetDefaultWidgetSettings( $params )
	{
		$r = array(
			'contacticons' => array(
				'label' => $this->T_('Contact icons'),
				'note' => $this->T_('Display contact icons allowing to send private messages to logged in users.'),
				'type' => 'checkbox',
				'defaultvalue' => true,
			),
			'timeout_online_user' => array(
				'label' => $this->T_('Online session timeout'),
				'note' => $this->T_('seconds. After how much time of inactivity an user is considered to be offline? 300 seconds are 5 minutes.'),
				'type' => 'integer',
				'defaultvalue' => 300,
				'valid_range' => array(
					'min' => 1, // 0 would not make sense.
				),
			),
		);
		return $r;
	}


	/**
	 * Event handler: TemplateTag (widget)
	 *
	 * @param array Associative array of parameters.
	 * @return boolean did we display?
	 */
	function TemplateTag( $params )
	{
		global $Plugins;

		if( ! $Plugins->trigger_event_first_true('CacheIsCollectingContent') )
		{
			// a caching plugin is not collecting the content, so we can display this block
			return false;
		}

		echo $params['block_start'];

		echo $params['block_title_start'];
		echo $this->T_('Who\'s Online?');
		echo $params['block_title_end'];

		$OnlineSessions = new OnlineSessions( $params['timeout_online_user'] );
		$OnlineSessions->display_onliners( $params );

		echo $params['block_end'];

		return true;
	}
}


/**
 * This tracks who is online
 *
 * @todo (0000) dh> I wanted to add a MySQL INDEX on the sess_lastseen field, but this "plugin"
 * is the only real user of this. So, when making this a plugin, this should add the index perhaps.
 *
 * @package plugins
 */
class OnlineSessions
{
	/**
	 * Number of guests (and users that want to be anonymous)
	 *
	 * Gets lazy-filled when needed, through {@link init()}.
	 */
	var $_count_guests;


	/**
	 * List of registered users.
	 *
	 * Gets lazy-filled when needed, through {@link init()}.
	 */
	var $_registered_Users;

	/**
	 * Online session timeout in seconds.
	 *
	 * Default value: 300 (5 minutes). Set by {@link OnlineSessions::OnlineSessions()}.
	 */
	var $_timeout_online_user;

	var $_initialized = false;

	/**
	 * A reference to the whosonline_plugin.
	 */
	var $Plugin;

	/**
	 * Constructor.
	 *
	 * @param integer Online session timeout in seconds.
	 */
	function OnlineSessions( $timeout_online_user = 300 )
	{
		global $Plugins;

		$this->_timeout_online_user = $timeout_online_user;
		$this->Plugin = & $Plugins->get_by_code( 'whosonline' );
	}


	/**
	 * Get an array of registered users and guests.
	 *
	 * @return array containing number of registered users and guests ('registered' and 'guests')
	 */
	function init()
	{
		if( $this->_initialized )
		{
			return true;
		}
		global $DB, $localtimenow;

		$this->_count_guests = 0;
		$this->_registered_Users = array();

		$timeout_YMD = date( 'Y-m-d H:i:s', ( $localtimenow - $this->_timeout_online_user ) );

		$UserCache = & get_Cache( 'UserCache' );

		// We get all sessions that have been seen in $timeout_YMD and that have a session key.
		// NOTE: we do not use DISTINCT here, because guest users are all "NULL".
		foreach( $DB->get_results( "
			SELECT sess_user_ID
			  FROM T_sessions
			 WHERE sess_lastseen > '".$timeout_YMD."'
			   AND sess_key IS NOT NULL", OBJECT, 'Sessions: get list of relevant users.' ) as $row )
		{
			if( ! empty( $row->sess_user_ID )
					&& ( $User = & $UserCache->get_by_ID( $row->sess_user_ID ) ) )
			{
				// assign by ID so that each user is only counted once (he could use multiple user agents at the same time)
				$this->_registered_Users[$User->ID] = & $User;

				if( !$User->showonline )
				{
					$this->_count_guests++;
				}
			}
			else
			{
				$this->_count_guests++;
			}
		}

		$this->_initialized = true;
	}


	/**
	 * Get the number of guests.
	 *
	 * @param boolean display?
	 */
	function number_of_guests( $display = true )
	{
		if( ! isset( $this->_count_guests ) )
		{
			$this->init();
		}

		if( $display )
		{
			echo $this->_count_guests;
		}
		return $this->_count_guests;
	}


	/**
	 * Template function: Display onliners, both registered users and guests.
	 *
	 * @todo (0000) get class="" out of here (put it into templates)
	 */
	function display_onliners( $params )
	{
		$this->display_online_users( $params );
		$this->display_online_guests( $params );
	}

	/**
	 * Template function: Display the registered users who are online
	 *
	 * @todo (0000) get class="" out of here (put it into templates)
	 *
	 * @param array
	 */
	function display_online_users( $params )
	{
		global $DB, $Blog;
		if( ! isset( $this->_registered_Users ) )
		{
			$this->init();
		}

		// Note: not all users want to appear online, so we might have an empty list.
		$r = '';

		foreach( $this->_registered_Users as $User )
		{
			if( $User->showonline )
			{
				if( empty( $r ) )
				{
					// first user
					$r .= $params['list_start'];
				}

				$r .= $params['item_start'];
				$r .= $User->dget( 'preferredname' );
				if( $params['contacticons'] )
				{
					// We want contact icons
					$r .= $User->get_msgform_link();
				}
				$r .= $params['item_end'];
			}
		}

		if( ! empty( $r ) )
		{
			// we need to close the list
			$r .= $params['list_end'];;
		}

		echo $r;
	}


	/**
	 * Template function: Display number of online guests.
	 */
	function display_online_guests( $params )
	{
		if( ! isset( $this->_count_guests ) )
		{
			$this->init();
		}

		$r = $params['list_start'];
		$r .= $params['item_start'];
		$r .= $this->Plugin->T_('Guest Users:').' ';
		$r .= $this->_count_guests;
		$r .= $params['item_end'];
		$r .= $params['list_end'];;

		echo $r;
	}
}

?>
